from trace_dump cimport TraceDump, FileTraceDump
from libc.stdio cimport FILE, fopen, fclose, stdout, stderr

cdef class PyTraceDump:
    cdef TraceDump* thisptr

    def __dealloc__(self):
        del self.thisptr

    def start(self):
        self.thisptr.start()

    def stop(self):
        self.thisptr.stop()


cdef class PyFileTraceDump(PyTraceDump):
    def __cinit__(self, const char* path):
        global trace_message_ring_buffer
        cdef FILE* handle = fopen(path, "wb")
        if handle == NULL:
            raise ValueError("cannot open file {}".format(path))
        self.thisptr = new FileTraceDump(trace_message_ring_buffer, handle, True)


cdef class PyStdoutTraceDump(PyTraceDump):
    def __cinit__(self):
        global trace_message_ring_buffer
        self.thisptr = new FileTraceDump(trace_message_ring_buffer, stdout, False)


cdef class PyStderrTraceDump(PyTraceDump):
    def __cinit__(self):
        global trace_message_ring_buffer
        self.thisptr = new FileTraceDump(trace_message_ring_buffer, stderr, False)


cdef class PySyslogTraceDump(PyTraceDump):
    def __cinit__(self):
        self.thisptr = NULL

cdef class PyWriter:
    cdef TraceMessageRingBuffer* ring_buffer
    cdef TraceDump* trace_dump

    def __cinit__(self):
        self.ring_buffer = NULL
        self.trace_dump = NULL

    def __dealloc__(self):
        self.trace_dump.stop()
        del self.trace_dump
        del self.ring_buffer

    def start(self):
        self.trace_dump.start()

    def stop(self):
        self.trace_dump.stop()

    def write(self, severity, message_str):
        cdef TraceMessage* message = self.ring_buffer.reserve_push()
        message.set_timestamp()
        message.set_severity(severity)
        message.write(message_str)
        self.ring_buffer.commit_push(message)


IF UNAME_SYSNAME != "Windows":
    include "ctracing_trace_dump_unix.pyx"



__version__ = "0.2.9"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(detached from v0.2.9)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = 'bd0ce8f8e66f63d54beabab7fdb3b0ad20991f87'
__git_head_subject__ = 'Finished release v0.2.9.'
__git_head_message__ = '* release/v0.2.9:\n  TRIVIAL gitignore\n  buildout.cfg: changed isolated python version to v2.7.6.21\n  buildout.cfg: changed isolated python version to v2.7.6.20\n  buildout.cfg: changed isolated python version to v2.7.6.19'
__git_dirty_diff__ = ''


cdef extern from "trace_dump.h":
    cdef cppclass SyslogSocket:
        SyslogSocket() nogil  # Cython limitation - see http://trac.cython.org/cython_trac/ticket/687

    cdef cppclass SyslogUNIXSocket(SyslogSocket):
        SyslogUNIXSocket() nogil  # Cython limitation - see http://trac.cython.org/cython_trac/ticket/687
        SyslogUNIXSocket(const char* address) nogil

    cdef cppclass SyslogTCPSocket(SyslogSocket):
        SyslogTCPSocket() nogil  # Cython limitation - see http://trac.cython.org/cython_trac/ticket/687
        SyslogTCPSocket(const char* address, int port) nogil

    # cdef cppclass SyslogUDPSocket(SyslogSocket):
    #     SyslogUDPSocket() nogil  # Cython limitation - see http://trac.cython.org/cython_trac/ticket/687
    #     SyslogUDPSocket(const char* address, int port) nogil

    cdef cppclass SyslogTraceDump(TraceDump):
        SyslogTraceDump() nogil  # Cython limitation - see http://trac.cython.org/cython_trac/ticket/687
        SyslogTraceDump(TraceMessageRingBuffer* ring_buffer, const char* host_name, const char* application_name,
                        const char* process_id, int facility, bool rfc5424, SyslogSocket* socket) nogil



from trace_dump cimport SyslogTraceDump, SyslogSocket, SyslogUNIXSocket, SyslogTCPSocket

cdef new_syslog_trace_dump(const char* host_name, const char* application_name, const char* process_id, int facility,
                           bool rfc5424, SyslogSocket* socket):
    global trace_message_ring_buffer
    cdef SyslogTraceDump* trace_dump = new SyslogTraceDump(trace_message_ring_buffer, host_name, application_name,
                                                           process_id, facility, rfc5424, socket)
    result = PySyslogTraceDump()
    result.thisptr = trace_dump
    return result


def PySyslogTraceDump_create_with_unix_socket(const char* host_name, const char* application_name,
                                              const char* process_id, int facility, bool rfc5424, const char* address):
    cdef SyslogUNIXSocket* socket = new SyslogUNIXSocket(address)
    return new_syslog_trace_dump(host_name, application_name, process_id, facility, rfc5424, socket)


def PySyslogTraceDump_create_with_tcp_socket(const char* host_name, const char* application_name,
                                             const char* process_id, int facility, bool rfc5424, const char* address,
                                             int port):
    cdef SyslogTCPSocket* socket = new SyslogTCPSocket(address, port)
    return new_syslog_trace_dump(host_name, application_name, process_id, facility, rfc5424, socket)

cdef new_syslog_writer(int buffer_size, int trace_message_size, const char* host_name, const char* application_name, 
                       const char* process_id, int facility, bool rfc5424, SyslogSocket* socket):
    cdef TraceMessageRingBuffer* ring_buffer = new TraceMessageRingBuffer(buffer_size, trace_message_size)
    cdef SyslogTraceDump* trace_dump = new SyslogTraceDump(ring_buffer, host_name, application_name, process_id,
                                                           facility, rfc5424, socket)
    result = PyWriter()
    result.ring_buffer = ring_buffer
    result.trace_dump = trace_dump
    return result

def PySyslogWriter_create_with_unix_socket(int buffer_size, int trace_message_size, const char* host_name, 
                                           const char* application_name, const char* process_id, int facility, 
                                           bool rfc5424, const char* address):
    cdef SyslogSocket* socket = new SyslogUNIXSocket(address)
    return new_syslog_writer(buffer_size, trace_message_size, host_name, application_name, process_id, facility, 
                             rfc5424, socket)


def PySyslogWriter_create_with_tcp_socket(int buffer_size, int trace_message_size, const char* host_name, 
                                          const char* application_name, const char* process_id, int facility, 
                                          bool rfc5424, const char* address, int port):
    cdef SyslogSocket* socket = new SyslogTCPSocket(address, port)
    return new_syslog_writer(buffer_size, trace_message_size, host_name, application_name, process_id, facility, 
                             rfc5424, socket)



__import__("pkg_resources").declare_namespace(__name__)
import sys
from infi.pyutils.contexts import contextmanager
from infi.pyutils.decorators import wraps

__all__ = ['set_tracing', 'unset_tracing', 'suspend_tracing', 'resume_tracing',
           'no_tracing_context_recursive', 'no_tracing_recursive',
           'tracing_output_to_file', 'tracing_output_to_stdout', 'tracing_output_to_stderr',
           'NO_TRACE', 'NO_TRACE_NESTED', 'TRACE_FUNC_NAME', 'TRACE_FUNC_PRIMITIVES', 'TRACE_FUNC_REPR',
           'wait_and_ensure_exit']
if sys.platform != 'win32':
    __all__.extend(['tracing_output_to_syslog', 'SyslogWriter'])

# Same as in ctracing:
NO_TRACE              = 0
NO_TRACE_NESTED       = 1
TRACE_FUNC_NAME       = 2
TRACE_FUNC_PRIMITIVES = 3
TRACE_FUNC_REPR       = 4

trace_dump = None


def _filter_all(*args, **kwargs):
    return True


def tracing_output_to_file(path):
    global trace_dump
    from infi.tracing.ctracing import PyFileTraceDump
    trace_dump = PyFileTraceDump(path)


def tracing_output_to_stdout():
    global trace_dump
    from infi.tracing.ctracing import PyStdoutTraceDump
    trace_dump = PyStdoutTraceDump()


def tracing_output_to_stderr():
    global trace_dump
    from infi.tracing.ctracing import PyStderrTraceDump
    trace_dump = PyStderrTraceDump()


def set_tracing(filter_func=_filter_all, trace_level_lru_capacity=256):
    global trace_dump
    from infi.tracing.ctracing import ctracing_set_profile
    if trace_dump:
        trace_dump.start()
    ctracing_set_profile(filter_func, trace_level_lru_capacity)


def unset_tracing():
    global trace_dump
    sys.setprofile(None)
    if trace_dump:
        trace_dump.stop()
        trace_dump = None


def wait_and_ensure_exit(seconds=0, exit_code=10):
    from infi.tracing.ctracing import ctracing_wait_and_ensure_exit
    ctracing_wait_and_ensure_exit(seconds, exit_code)


def suspend_tracing():
    from infi.tracing.ctracing import suspend
    suspend()


def resume_tracing():
    from infi.tracing.ctracing import resume
    resume()


@contextmanager
def no_tracing_context_recursive():
    try:
        suspend_tracing()
        yield
    finally:
        resume_tracing()


def no_tracing_recursive(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        with no_tracing_context_recursive():
            return func(*args, **kwargs)
    return wrapper


if sys.platform != 'win32':
    def default_syslog_address():
        import os
        import sys
        if sys.platform == 'darwin':
            address = '/var/run/syslog'
        else:
            address = '/dev/log'
        if not os.path.exists(address):
            raise ValueError("cannot find syslog socket address: {}".format(address))
        return address

    def _check_syslog_application_name_and_facility(application_name, facility):
        if not isinstance(application_name, (str, unicode)):
            raise TypeError("application_name must be a string or unicode object")

        if not isinstance(facility, int):
            raise TypeError("facility must be an int object but instead is {}".format(type(facility)))

    def _check_syslog_address(address):
        import os
        if isinstance(address, (str, unicode)):
            # UNIX socket:
            if not os.path.exists(address):
                raise ValueError("cannot find syslog socket address: {}".format(address))
            return 'unix'
        else:
            import re
            if not isinstance(address, (tuple, list)):
                raise TypeError("syslog address must be a string or an (addr, port) pair")
            if not re.match(r'\d+\.\d+\.\d+\.\d+', address[0]):
                raise ValueError("syslog address first element must be an IP address but got {!r}".format(address[0]))
            return 'tcp'

    def SyslogWriter(buffer_size, trace_message_size, facility, address=None, host_name="", application_name="",
                     process_id="", rfc5424=False):
        from infi.tracing.ctracing import (PySyslogWriter_create_with_unix_socket, PySyslogWriter_create_with_tcp_socket)
        _check_syslog_application_name_and_facility(application_name, facility)
        if address is None:
            address = default_syslog_address()
        if _check_syslog_address(address) == 'tcp':
            return PySyslogWriter_create_with_tcp_socket(buffer_size, trace_message_size, host_name, application_name,
                                                         process_id, facility, bool(rfc5424), address[0], address[1])
        else:
            return PySyslogWriter_create_with_unix_socket(buffer_size, trace_message_size, host_name, application_name,
                                                          process_id, facility, bool(rfc5424), address)

    def tracing_output_to_syslog(facility, host_name="", application_name="", process_id="", address=None, rfc5424=False):
        global trace_dump
        from infi.tracing.ctracing import (PySyslogTraceDump_create_with_unix_socket,
                                           PySyslogTraceDump_create_with_tcp_socket)
        _check_syslog_application_name_and_facility(application_name, facility)
        if address is None:
            address = default_syslog_address()
        if (_check_syslog_address(address) == 'tcp'):
            trace_dump = PySyslogTraceDump_create_with_tcp_socket(host_name, application_name, process_id, facility,
                                                                  bool(rfc5424), address[0], address[1])
        else:
            trace_dump = PySyslogTraceDump_create_with_unix_socket(host_name, application_name, process_id, facility,
                                                                   bool(rfc5424), address)



from libcpp cimport bool
from trace_message cimport TraceMessage

cdef extern from "trace_message_ring_buffer.h":
    cdef cppclass TraceMessageRingBuffer:
        TraceMessageRingBuffer(size_t capacity, size_t trace_message_capacity) nogil

        TraceMessage* reserve_push() nogil
        void commit_push(TraceMessage* message) nogil

        unsigned long get_overflow_counter() nogil
        unsigned long get_and_reset_overflow_counter() nogil
        unsigned long get_spinlock_consumer_wait_counter() nogil
        unsigned long get_spinlock_producer_wait_counter() nogil